@@ -1,15 +1,15 @@
 // Imports
-use super::{Engine, EngineConfig, StrokeContent};
+use super::{Engine, StrokeContent};
 use crate::fileformats::rnoteformat::RnoteFile;
-use crate::fileformats::{xoppformat, FileFormatSaver};
-use crate::CloneConfig;
+use crate::fileformats::{FileFormatSaver, xoppformat};
 use anyhow::Context;
 use futures::channel::oneshot;
 use rayon::prelude::*;
-use rnote_compose::transform::Transformable;
 use rnote_compose::SplitOrder;
+use rnote_compose::transform::Transformable;
 use serde::{Deserialize, Serialize};
 use std::sync::Arc;
+use tracing::error;
 
 /// Document export format.
 #[derive(
@@ -311,12 +311,6 @@ pub struct ExportPrefs {
     pub selection_export_prefs: SelectionExportPrefs,
 }
 
-impl CloneConfig for ExportPrefs {
-    fn clone_config(&self) -> Self {
-        *self
-    }
-}
-
 impl Engine {
     /// The used image scale-factor for any strokes that are converted to bitmap images on export.
     pub const STROKE_EXPORT_IMAGE_SCALE: f64 = 1.8;
@@ -336,7 +330,7 @@ impl Engine {
                 rnote_file.save_as_bytes(&file_name)
             };
             if oneshot_sender.send(result()).is_err() {
-                tracing::error!(
+                error!(
                     "Sending result to receiver failed while saving document as rnote bytes. Receiver already dropped."
                 );
             }
@@ -344,18 +338,6 @@ impl Engine {
         oneshot_receiver
     }
 
-    /// Extract the current engine configuration.
-    pub fn extract_engine_config(&self) -> EngineConfig {
-        EngineConfig {
-            document: self.document.clone_config(),
-            pens_config: self.pens_config.clone_config(),
-            penholder: self.penholder.clone_config(),
-            import_prefs: self.import_prefs.clone_config(),
-            export_prefs: self.export_prefs.clone_config(),
-            pen_sounds: self.pen_sounds(),
-        }
-    }
-
     pub fn extract_document_content(&self) -> StrokeContent {
         StrokeContent::default()
             .with_strokes(
@@ -366,7 +348,7 @@ impl Engine {
                 self.bounds_w_content_extended()
                     .unwrap_or(self.document.bounds()),
             ))
-            .with_background(Some(self.document.background))
+            .with_background(Some(self.document.config.background))
     }
 
     pub fn extract_pages_content(&self, page_order: SplitOrder) -> Vec<StrokeContent> {
@@ -382,7 +364,7 @@ impl Engine {
                         ),
                     )
                     .with_bounds(Some(bounds))
-                    .with_background(Some(self.document.background))
+                    .with_background(Some(self.document.config.background))
             })
             .collect()
     }
@@ -395,15 +377,10 @@ impl Engine {
         Some(
             StrokeContent::default()
                 .with_strokes(self.store.get_strokes_arc(&selection_keys))
-                .with_background(Some(self.document.background)),
+                .with_background(Some(self.document.config.background)),
         )
     }
 
-    /// Export the current engine config as Json string.
-    pub fn export_engine_config_as_json(&self) -> anyhow::Result<String> {
-        Ok(serde_json::to_string(&self.extract_engine_config())?)
-    }
-
     /// Export the entire engine state as Json string.
     ///
     /// Only intended to be used for debugging.
@@ -418,7 +395,7 @@ impl Engine {
         doc_export_prefs_override: Option<DocExportPrefs>,
     ) -> oneshot::Receiver<Result<Vec<u8>, anyhow::Error>> {
         let doc_export_prefs =
-            doc_export_prefs_override.unwrap_or(self.export_prefs.doc_export_prefs);
+            doc_export_prefs_override.unwrap_or(self.config.read().export_prefs.doc_export_prefs);
 
         match doc_export_prefs.export_format {
             DocExportFormat::Svg => self.export_doc_as_svg_bytes(doc_export_prefs_override),
@@ -436,7 +413,7 @@ impl Engine {
     ) -> oneshot::Receiver<Result<Vec<u8>, anyhow::Error>> {
         let (oneshot_sender, oneshot_receiver) = oneshot::channel::<anyhow::Result<Vec<u8>>>();
         let doc_export_prefs =
-            doc_export_prefs_override.unwrap_or(self.export_prefs.doc_export_prefs);
+            doc_export_prefs_override.unwrap_or(self.config.read().export_prefs.doc_export_prefs);
         let doc_content = self.extract_document_content();
 
         rayon::spawn(move || {
@@ -462,7 +439,9 @@ impl Engine {
             };
 
             if oneshot_sender.send(result()).is_err() {
-                tracing::error!("Sending result to receiver failed while exporting document as Svg bytes. Receiver already dropped.");
+                error!(
+                    "Sending result to receiver failed while exporting document as Svg bytes. Receiver already dropped."
+                );
             }
         });
 
@@ -477,9 +456,9 @@ impl Engine {
     ) -> oneshot::Receiver<anyhow::Result<Vec<u8>>> {
         let (oneshot_sender, oneshot_receiver) = oneshot::channel::<anyhow::Result<Vec<u8>>>();
         let doc_export_prefs =
-            doc_export_prefs_override.unwrap_or(self.export_prefs.doc_export_prefs);
+            doc_export_prefs_override.unwrap_or(self.config.read().export_prefs.doc_export_prefs);
         let pages_content = self.extract_pages_content(doc_export_prefs.page_order);
-        let format_size = self.document.format.size();
+        let format_size = self.document.config.format.size();
 
         rayon::spawn(move || {
             let result = || -> anyhow::Result<Vec<u8>> {
@@ -536,7 +515,9 @@ impl Engine {
             };
 
             if oneshot_sender.send(result()).is_err() {
-                tracing::error!("Sending result to receiver failed while exporting document as Pdf bytes. Receiver already dropped.");
+                error!(
+                    "Sending result to receiver failed while exporting document as Pdf bytes. Receiver already dropped."
+                );
             }
         });
 
@@ -551,7 +532,7 @@ impl Engine {
     ) -> oneshot::Receiver<Result<Vec<u8>, anyhow::Error>> {
         let (oneshot_sender, oneshot_receiver) = oneshot::channel::<anyhow::Result<Vec<u8>>>();
         let doc_export_prefs =
-            doc_export_prefs_override.unwrap_or(self.export_prefs.doc_export_prefs);
+            doc_export_prefs_override.unwrap_or(self.config.read().export_prefs.doc_export_prefs);
         let pages_content = self.extract_pages_content(doc_export_prefs.page_order);
         let document = self.document.clone();
 
@@ -561,7 +542,7 @@ impl Engine {
                 let xopp_background = xoppformat::XoppBackground {
                     name: None,
                     bg_type: xoppformat::XoppBackgroundType::Solid {
-                        color: crate::utils::xoppcolor_from_color(document.background.color),
+                        color: crate::utils::xoppcolor_from_color(document.config.background.color),
                         style: xoppformat::XoppBackgroundSolidStyle::Plain,
                     },
                 };
@@ -579,7 +560,7 @@ impl Engine {
                             .filter_map(|mut stroke| {
                                 let mut stroke = Arc::make_mut(&mut stroke).clone();
                                 stroke.translate(-page_bounds.mins.coords);
-                                stroke.into_xopp(document.format.dpi())
+                                stroke.into_xopp(document.config.format.dpi())
                             })
                             .collect::<Vec<xoppformat::XoppStrokeType>>();
 
@@ -637,7 +618,7 @@ impl Engine {
 
                         let page_dimensions = crate::utils::convert_coord_dpi(
                             page_bounds.extents(),
-                            document.format.dpi(),
+                            document.config.format.dpi(),
                             xoppformat::XoppFile::DPI,
                         );
 
@@ -651,7 +632,7 @@ impl Engine {
                     .collect::<Vec<xoppformat::XoppPage>>();
 
                 let xopp_title = String::from(
-                    "Xournal++ document - see https://github.com/xournalpp/xournalpp (exported from Rnote - see https://github.com/flxzt/rnote)"
+                    "Xournal++ document - see https://github.com/xournalpp/xournalpp (exported from Rnote - see https://github.com/flxzt/rnote)",
                 );
 
                 let xopp_root = xoppformat::XoppRoot {
@@ -666,7 +647,7 @@ impl Engine {
             };
 
             if oneshot_sender.send(result()).is_err() {
-                tracing::error!(
+                error!(
                     "Sending result to receiver failed while exporting document as xopp bytes. Receiver already dropped."
                 );
             }
@@ -680,8 +661,8 @@ impl Engine {
         &self,
         doc_pages_export_prefs_override: Option<DocPagesExportPrefs>,
     ) -> oneshot::Receiver<Result<Vec<Vec<u8>>, anyhow::Error>> {
-        let doc_pages_export_prefs =
-            doc_pages_export_prefs_override.unwrap_or(self.export_prefs.doc_pages_export_prefs);
+        let doc_pages_export_prefs = doc_pages_export_prefs_override
+            .unwrap_or(self.config.read().export_prefs.doc_pages_export_prefs);
 
         match doc_pages_export_prefs.export_format {
             DocPagesExportFormat::Svg => {
@@ -699,8 +680,8 @@ impl Engine {
         doc_pages_export_prefs_override: Option<DocPagesExportPrefs>,
     ) -> oneshot::Receiver<Result<Vec<Vec<u8>>, anyhow::Error>> {
         let (oneshot_sender, oneshot_receiver) = oneshot::channel::<anyhow::Result<Vec<Vec<u8>>>>();
-        let doc_pages_export_prefs =
-            doc_pages_export_prefs_override.unwrap_or(self.export_prefs.doc_pages_export_prefs);
+        let doc_pages_export_prefs = doc_pages_export_prefs_override
+            .unwrap_or(self.config.read().export_prefs.doc_pages_export_prefs);
         let pages_content = self.extract_pages_content(doc_pages_export_prefs.page_order);
 
         rayon::spawn(move || {
@@ -734,7 +715,7 @@ impl Engine {
             };
 
             if oneshot_sender.send(result()).is_err() {
-                tracing::error!(
+                error!(
                     "Sending result to receiver failed while exporting document pages as Svg bytes. Receiver already dropped."
                 );
             }
@@ -751,18 +732,20 @@ impl Engine {
         doc_pages_export_prefs_override: Option<DocPagesExportPrefs>,
     ) -> oneshot::Receiver<Result<Vec<Vec<u8>>, anyhow::Error>> {
         let (oneshot_sender, oneshot_receiver) = oneshot::channel::<anyhow::Result<Vec<Vec<u8>>>>();
-        let doc_pages_export_prefs =
-            doc_pages_export_prefs_override.unwrap_or(self.export_prefs.doc_pages_export_prefs);
+        let doc_pages_export_prefs = doc_pages_export_prefs_override
+            .unwrap_or(self.config.read().export_prefs.doc_pages_export_prefs);
         let pages_contents = self.extract_pages_content(doc_pages_export_prefs.page_order);
 
         rayon::spawn(move || {
             let result = || -> Result<Vec<Vec<u8>>, anyhow::Error> {
                 let image_format = match doc_pages_export_prefs.export_format {
-                    DocPagesExportFormat::Svg => return Err(anyhow::anyhow!("Extracting bitmap image format from doc pages export prefs failed, not set to a bitmap format.")),
-                    DocPagesExportFormat::Png => image::ImageOutputFormat::Png,
-                    DocPagesExportFormat::Jpeg => {
-                        image::ImageOutputFormat::Jpeg(doc_pages_export_prefs.jpeg_quality)
+                    DocPagesExportFormat::Svg => {
+                        return Err(anyhow::anyhow!(
+                            "Extracting bitmap image format from doc pages export prefs failed, not set to a bitmap format."
+                        ));
                     }
+                    DocPagesExportFormat::Png => image::ImageFormat::Png,
+                    DocPagesExportFormat::Jpeg => image::ImageFormat::Jpeg,
                 };
                 pages_contents
                     .into_par_iter()
@@ -779,12 +762,17 @@ impl Engine {
                                 "Generating Svg for page {i} failed, returned None."
                             ))?
                             .gen_image(doc_pages_export_prefs.bitmap_scalefactor)?
-                            .into_encoded_bytes(image_format.clone())
+                            .into_encoded_bytes(
+                                image_format,
+                                Some(doc_pages_export_prefs.jpeg_quality),
+                            )
                     })
                     .collect()
             };
             if oneshot_sender.send(result()).is_err() {
-                tracing::error!("Sending result to receiver failed while exporting document pages as bitmap bytes. Receiver already dropped.");
+                error!(
+                    "Sending result to receiver failed while exporting document pages as bitmap bytes. Receiver already dropped."
+                );
             }
         });
 
@@ -796,8 +784,8 @@ impl Engine {
         &self,
         selection_export_prefs_override: Option<SelectionExportPrefs>,
     ) -> oneshot::Receiver<Result<Option<Vec<u8>>, anyhow::Error>> {
-        let selection_export_prefs =
-            selection_export_prefs_override.unwrap_or(self.export_prefs.selection_export_prefs);
+        let selection_export_prefs = selection_export_prefs_override
+            .unwrap_or(self.config.read().export_prefs.selection_export_prefs);
 
         match selection_export_prefs.export_format {
             SelectionExportFormat::Svg => {
@@ -816,8 +804,8 @@ impl Engine {
     ) -> oneshot::Receiver<Result<Option<Vec<u8>>, anyhow::Error>> {
         let (oneshot_sender, oneshot_receiver) =
             oneshot::channel::<anyhow::Result<Option<Vec<u8>>>>();
-        let selection_export_prefs =
-            selection_export_prefs_override.unwrap_or(self.export_prefs.selection_export_prefs);
+        let selection_export_prefs = selection_export_prefs_override
+            .unwrap_or(self.config.read().export_prefs.selection_export_prefs);
         let content = self.extract_selection_content();
 
         rayon::spawn(move || {
@@ -849,7 +837,9 @@ impl Engine {
                 ))
             };
             if oneshot_sender.send(result()).is_err() {
-                tracing::error!("Sending result to receiver failed while exporting selection as Svg bytes. Receiver already dropped.");
+                error!(
+                    "Sending result to receiver failed while exporting selection as Svg bytes. Receiver already dropped."
+                );
             }
         });
 
@@ -865,8 +855,8 @@ impl Engine {
     ) -> oneshot::Receiver<Result<Option<Vec<u8>>, anyhow::Error>> {
         let (oneshot_sender, oneshot_receiver) =
             oneshot::channel::<anyhow::Result<Option<Vec<u8>>>>();
-        let selection_export_prefs =
-            selection_export_prefs_override.unwrap_or(self.export_prefs.selection_export_prefs);
+        let selection_export_prefs = selection_export_prefs_override
+            .unwrap_or(self.config.read().export_prefs.selection_export_prefs);
         let content = self.extract_selection_content();
 
         rayon::spawn(move || {
@@ -884,20 +874,27 @@ impl Engine {
                     return Ok(None);
                 };
                 let image_format = match selection_export_prefs.export_format {
-                    SelectionExportFormat::Svg => return Err(anyhow::anyhow!("Extracting bitmap image format from doc pages export prefs failed, not set to a bitmap format.")),
-                    SelectionExportFormat::Png => image::ImageOutputFormat::Png,
-                    SelectionExportFormat::Jpeg => {
-                        image::ImageOutputFormat::Jpeg(selection_export_prefs.jpeg_quality)
+                    SelectionExportFormat::Svg => {
+                        return Err(anyhow::anyhow!(
+                            "Extracting bitmap image format from doc pages export prefs failed, not set to a bitmap format."
+                        ));
                     }
+                    SelectionExportFormat::Png => image::ImageFormat::Png,
+                    SelectionExportFormat::Jpeg => image::ImageFormat::Jpeg,
                 };
 
                 Ok(Some(
                     svg.gen_image(selection_export_prefs.bitmap_scalefactor)?
-                        .into_encoded_bytes(image_format)?,
+                        .into_encoded_bytes(
+                            image_format,
+                            Some(selection_export_prefs.jpeg_quality),
+                        )?,
                 ))
             };
             if oneshot_sender.send(result()).is_err() {
-                tracing::error!("Sending result to receiver failed while exporting selection as bitmap image bytes. Receiver already dropped");
+                error!(
+                    "Sending result to receiver failed while exporting selection as bitmap image bytes. Receiver already dropped"
+                );
             }
         });
 
