@@ -2,14 +2,13 @@
 use super::penmode::PenModeState;
 use super::shortcuts::ShortcutMode;
 use super::{
-    Brush, Eraser, Pen, PenBehaviour, PenMode, PenStyle, Selector, Shaper, Shortcuts, Tools,
-    Typewriter,
+    Brush, Eraser, Pen, PenBehaviour, PenMode, PenStyle, Selector, Shaper, Tools, Typewriter,
 };
+use crate::DrawableOnDoc;
 use crate::camera::NudgeDirection;
 use crate::engine::{EngineView, EngineViewMut};
 use crate::pens::shortcuts::ShortcutAction;
 use crate::widgetflags::WidgetFlags;
-use crate::{CloneConfig, DrawableOnDoc};
 use futures::channel::oneshot;
 use p2d::bounding_volume::Aabb;
 use piet::RenderContext;
@@ -18,24 +17,25 @@ use rnote_compose::penevent::{KeyboardKey, ModifierKey, PenEvent, PenProgress, S
 use serde::{Deserialize, Serialize};
 use std::time::{Duration, Instant};
 
-#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Serialize, Deserialize)]
+#[serde(rename = "backlog_policy")]
 pub enum BacklogPolicy {
+    #[serde(rename = "no_limit")]
     NoLimit,
+    #[serde(rename = "limit")]
     Limit(Duration),
-    DisableBacklog,
+    #[serde(rename = "disable")]
+    Disable,
 }
 
 /// The Penholder holds the pens and related state and handles pen events.
 #[derive(Debug, Serialize, Deserialize)]
 #[serde(default, rename = "penholder")]
 pub struct PenHolder {
-    #[serde(rename = "shortcuts")]
-    shortcuts: Shortcuts,
     #[serde(rename = "pen_mode_state")]
     pen_mode_state: PenModeState,
-
     /// The policy for the retrieval of input event backlogs.
-    #[serde(skip)]
+    #[serde(rename = "backlog_policy")]
     backlog_policy: BacklogPolicy,
     #[serde(skip)]
     current_pen: Pen,
@@ -50,7 +50,6 @@ pub struct PenHolder {
 impl Default for PenHolder {
     fn default() -> Self {
         Self {
-            shortcuts: Shortcuts::default(),
             pen_mode_state: PenModeState::default(),
             backlog_policy: BacklogPolicy::NoLimit,
 
@@ -62,55 +61,7 @@ impl Default for PenHolder {
     }
 }
 
-impl CloneConfig for PenHolder {
-    fn clone_config(&self) -> Self {
-        Self {
-            shortcuts: self.shortcuts.clone(),
-            pen_mode_state: self.pen_mode_state.clone_config(),
-            ..Default::default()
-        }
-    }
-}
-
 impl PenHolder {
-    /// Get the current registered shortcuts.
-    pub fn shortcuts(&self) -> Shortcuts {
-        self.shortcuts.clone()
-    }
-
-    /// Clear all shortcuts
-    pub fn clear_shortcuts(&mut self) {
-        self.shortcuts.clear();
-    }
-
-    /// Replace all shortcuts.
-    pub fn set_shortcuts(&mut self, shortcuts: Shortcuts) {
-        self.shortcuts = shortcuts;
-    }
-
-    /// Register a shortcut key and action.
-    pub fn register_shortcut(&mut self, key: ShortcutKey, action: ShortcutAction) {
-        self.shortcuts.insert(key, action);
-    }
-
-    /// Remove the shortcut action for the given shortcut key, if it is registered.
-    pub fn remove_shortcut(&mut self, key: ShortcutKey) -> Option<ShortcutAction> {
-        self.shortcuts.remove(&key)
-    }
-
-    // Get the current registered action the the given shortcut key.
-    pub fn get_shortcut_action(&self, key: ShortcutKey) -> Option<ShortcutAction> {
-        self.shortcuts.get(&key).cloned()
-    }
-
-    /// List all current registered shortcut keys and their action.
-    pub fn list_current_shortcuts(&self) -> Vec<(ShortcutKey, ShortcutAction)> {
-        self.shortcuts
-            .iter()
-            .map(|(key, action)| (*key, *action))
-            .collect()
-    }
-
     /// Get the current pen mode state.
     pub fn pen_mode_state(&self) -> PenModeState {
         self.pen_mode_state.clone()
@@ -126,13 +77,14 @@ impl PenHolder {
     }
 
     /// Get the style without the temporary override.
-    pub fn current_pen_style(&self) -> PenStyle {
-        self.pen_mode_state.style()
+    pub fn current_pen_style(&self, engine_view: &EngineView) -> PenStyle {
+        self.pen_mode_state.style(&engine_view.config.pens_config)
     }
 
     /// Get the current style, or the override if it is set.
-    pub fn current_pen_style_w_override(&self) -> PenStyle {
-        self.pen_mode_state.current_style_w_override()
+    pub fn current_pen_style_w_override(&self, engine_view: &EngineView) -> PenStyle {
+        self.pen_mode_state
+            .current_style_w_override(&engine_view.config.pens_config)
     }
 
     /// The current pen progress.
@@ -214,10 +166,10 @@ impl PenHolder {
                 .handle_event(PenEvent::Cancel, Instant::now(), engine_view);
 
         // then reinstall a new pen instance
-        let mut new_pen = new_pen(self.current_pen_style_w_override());
+        let mut new_pen = new_pen(self.current_pen_style_w_override(&engine_view.as_im()));
         widget_flags |= new_pen.init(&engine_view.as_im()) | new_pen.update_state(engine_view);
         self.current_pen = new_pen;
-        widget_flags |= self.handle_changed_pen_style();
+        widget_flags |= self.handle_changed_pen_style(engine_view);
         self.progress = PenProgress::Idle;
 
         widget_flags
@@ -253,12 +205,21 @@ impl PenHolder {
             widget_flags |= wf;
         }
 
-        // Always redraw after handling a pen event
+        // Always redraw after handling a pen event.
+        //
+        // This is also needed because pens might have claimed/requested an animation frame.
         widget_flags.redraw = true;
 
         (event_result.propagate, widget_flags)
     }
 
+    /// Handle a requested animation frame.
+    ///
+    /// Can request another frame using `EngineViewMut#animation.claim_frame()`.
+    pub fn handle_animation_frame(&mut self, engine_view: &mut EngineViewMut) {
+        self.current_pen.handle_animation_frame(engine_view);
+    }
+
     /// Handle a pressed shortcut key.
     pub fn handle_pressed_shortcut_key(
         &mut self,
@@ -269,7 +230,11 @@ impl PenHolder {
         let mut widget_flags = WidgetFlags::default();
         let mut propagate = EventPropagation::Proceed;
 
-        if let Some(action) = self.get_shortcut_action(shortcut_key) {
+        if let Some(action) = engine_view
+            .config
+            .pens_config
+            .get_shortcut_action(shortcut_key)
+        {
             match action {
                 ShortcutAction::ChangePenStyle { style, mode } => match mode {
                     ShortcutMode::Temporary => {
@@ -295,10 +260,12 @@ impl PenHolder {
                                     self.change_style_int(toggle_pen_style, engine_view);
                             }
                         } else {
-                            self.toggle_pen_style = Some(self.current_pen_style());
+                            self.toggle_pen_style =
+                                Some(self.current_pen_style(&engine_view.as_im()));
                             widget_flags |= self.change_style_int(style, engine_view);
                         }
                     }
+                    ShortcutMode::Disabled => {}
                 },
             }
 
@@ -338,12 +305,13 @@ impl PenHolder {
     ) -> WidgetFlags {
         let mut widget_flags = WidgetFlags::default();
 
-        if self.pen_mode_state.style() != new_style {
+        if self.pen_mode_state.style(&engine_view.config.pens_config) != new_style {
             // Deselecting when changing the style
             let all_strokes = engine_view.store.selection_keys_as_rendered();
             engine_view.store.set_selected_keys(&all_strokes, false);
 
-            self.pen_mode_state.set_style(new_style);
+            self.pen_mode_state
+                .set_style(&mut engine_view.config.pens_config, new_style);
             widget_flags |= self.reinstall_pen_current_style(engine_view);
             widget_flags.refresh_ui = true;
         }
@@ -474,9 +442,8 @@ impl PenHolder {
                 // take the style override when pen is finished
                 if self.pen_mode_state.take_style_override().is_some() {
                     widget_flags.refresh_ui = true;
+                    widget_flags |= self.reinstall_pen_current_style(engine_view);
                 }
-
-                widget_flags |= self.reinstall_pen_current_style(engine_view);
             }
         }
 
@@ -485,9 +452,11 @@ impl PenHolder {
         widget_flags
     }
 
-    fn handle_changed_pen_style(&mut self) -> WidgetFlags {
+    fn handle_changed_pen_style(&mut self, engine_view: &mut EngineViewMut) -> WidgetFlags {
         let mut widget_flags = WidgetFlags::default();
-        let current_style = self.pen_mode_state.current_style_w_override();
+        let current_style = self
+            .pen_mode_state
+            .current_style_w_override(&engine_view.config.pens_config);
 
         self.backlog_policy = match current_style {
             PenStyle::Brush => BacklogPolicy::Limit(Duration::from_millis(4)),
@@ -495,7 +464,7 @@ impl PenHolder {
             PenStyle::Typewriter => BacklogPolicy::Limit(Duration::from_millis(33)),
             PenStyle::Eraser => BacklogPolicy::Limit(Duration::from_millis(33)),
             PenStyle::Selector => BacklogPolicy::Limit(Duration::from_millis(33)),
-            PenStyle::Tools => BacklogPolicy::DisableBacklog,
+            PenStyle::Tools => BacklogPolicy::Disable,
         };
 
         // Enable text preprocessing for typewriter
