@@ -1,7 +1,8 @@
 // Imports
 use crate::Drawable;
 use anyhow::Context;
-use image::io::Reader;
+use core::fmt::Debug;
+use image::ImageReader;
 use once_cell::sync::Lazy;
 use p2d::bounding_volume::{Aabb, BoundingVolume};
 use piet::RenderContext;
@@ -10,13 +11,14 @@ use rnote_compose::shapes::{Rectangle, Shapeable};
 use rnote_compose::transform::Transformable;
 use serde::{Deserialize, Serialize};
 use std::io::{self, Cursor};
+use std::sync::Arc;
 use svg::Node;
 
 /// Usvg font database
-pub static USVG_FONTDB: Lazy<usvg::fontdb::Database> = Lazy::new(|| {
+pub static USVG_FONTDB: Lazy<Arc<usvg::fontdb::Database>> = Lazy::new(|| {
     let mut db = usvg::fontdb::Database::new();
     db.load_system_fonts();
-    db
+    Arc::new(db)
 });
 
 /// Px unit (96 DPI ) to Point unit ( 72 DPI ) conversion factor.
@@ -76,7 +78,7 @@ impl From<ImageMemoryFormat> for piet::ImageFormat {
 }
 
 /// A bitmap image.
-#[derive(Debug, Clone, Serialize, Deserialize)]
+#[derive(Clone, Serialize, Deserialize)]
 #[serde(default, rename = "image")]
 pub struct Image {
     /// The image data.
@@ -98,6 +100,18 @@ pub struct Image {
     pub memory_format: ImageMemoryFormat,
 }
 
+impl Debug for Image {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Image")
+            .field("data", &String::from("{.. no debug impl ..}"))
+            .field("rect", &self.rect)
+            .field("pixel_width", &self.pixel_width)
+            .field("pixel_height", &self.pixel_height)
+            .field("memory_format", &self.memory_format)
+            .finish()
+    }
+}
+
 impl Default for Image {
     fn default() -> Self {
         Self {
@@ -136,7 +150,7 @@ impl Drawable for Image {
     ///
     /// Expects image to be in rgba8-premultiplied format, else drawing will fail.
     ///
-    /// `image_scale` has no meaning here, because the bitamp is already provided.
+    /// `image_scale` has no meaning here, because the bitmap is already provided.
     fn draw(&self, cx: &mut impl piet::RenderContext, _image_scale: f64) -> anyhow::Result<()> {
         let piet_image_format = piet::ImageFormat::from(self.memory_format);
 
@@ -191,7 +205,7 @@ impl Image {
     }
 
     pub fn try_from_encoded_bytes(bytes: &[u8]) -> Result<Self, anyhow::Error> {
-        let reader = Reader::new(io::Cursor::new(bytes)).with_guessed_format()?;
+        let reader = ImageReader::new(io::Cursor::new(bytes)).with_guessed_format()?;
         Ok(Image::from(reader.decode()?))
     }
 
@@ -219,31 +233,51 @@ impl Image {
         self.assert_valid()?;
 
         match self.memory_format {
-            ImageMemoryFormat::R8g8b8a8Premultiplied => {
-                image::RgbaImage::from_vec(self.pixel_width, self.pixel_height, self.data.to_vec())
-                    .ok_or_else(|| {
-                        anyhow::anyhow!(
+            ImageMemoryFormat::R8g8b8a8Premultiplied => image::RgbaImage::from_vec(
+                self.pixel_width,
+                self.pixel_height,
+                self.data.to_vec(),
+            )
+            .ok_or_else(|| {
+                anyhow::anyhow!(
                     "Creating RgbaImage from data failed for image with memory-format {:?}.",
                     self.memory_format
                 )
-                    })
-            }
+            }),
         }
     }
 
+    /// Encodes the image into the provided format.
+    ///
+    /// When the format is `Jpeg`, the quality should be provided, but falls back to 93 if it is None.
     pub fn into_encoded_bytes(
         self,
-        format: image::ImageOutputFormat,
+        format: image::ImageFormat,
+        quality: Option<u8>,
     ) -> Result<Vec<u8>, anyhow::Error> {
+        const QUALITY_FALLBACK: u8 = 93;
+
         self.assert_valid()?;
         let mut bytes_buf: Cursor<Vec<u8>> = Cursor::new(Vec::new());
         let dynamic_image = image::DynamicImage::ImageRgba8(
             self.into_imgbuf()
                 .context("Converting image to image::ImageBuffer failed.")?,
         );
-        dynamic_image
-            .write_to(&mut bytes_buf, format)
-            .context("Writing dynamic image to bytes buffer failed.")?;
+        match format {
+            image::ImageFormat::Jpeg => {
+                image::codecs::jpeg::JpegEncoder::new_with_quality(
+                    &mut bytes_buf,
+                    quality.map(|q| q.clamp(0, 100)).unwrap_or(QUALITY_FALLBACK),
+                )
+                .encode_image(&dynamic_image)
+                .context("Encode dynamic image to jpeg failed.")?;
+            }
+            format => {
+                dynamic_image
+                    .write_to(&mut bytes_buf, format)
+                    .context("Encode dynamic image to format '{format}' failed.")?;
+            }
+        }
 
         Ok(bytes_buf.into_inner())
     }
@@ -436,8 +470,13 @@ impl Svg {
             false,
         );
 
-        let usvg_tree =
-            usvg::Tree::from_str(&svg_data_wrapped, &usvg::Options::default(), &USVG_FONTDB)?;
+        let usvg_tree = usvg::Tree::from_str(
+            &svg_data_wrapped,
+            &usvg::Options {
+                fontdb: Arc::clone(&USVG_FONTDB),
+                ..Default::default()
+            },
+        )?;
 
         self.svg_data = usvg_tree.to_string(&xml_options);
         self.bounds = bounds_simplified;
