@@ -1,17 +1,19 @@
 // Imports
-use super::render_comp::RenderCompState;
 use super::StrokeKey;
+use super::render_comp::RenderCompState;
 use crate::engine::StrokeContent;
 use crate::strokes::{Content, Stroke};
 use crate::{StrokeStore, WidgetFlags};
 use geo::intersects::Intersects;
 use geo::prelude::Contains;
 use p2d::bounding_volume::{Aabb, BoundingVolume};
+use rnote_compose::Color;
 use rnote_compose::penpath::Element;
 use rnote_compose::shapes::Shapeable;
 use rnote_compose::transform::Transformable;
-use rnote_compose::Color;
 use std::sync::Arc;
+#[cfg(feature = "ui")]
+use tracing::error;
 
 /// Systems that are related to the stroke components.
 impl StrokeStore {
@@ -224,7 +226,7 @@ impl StrokeStore {
                     Ok(rendernodes) => {
                         render_comp.rendernodes = rendernodes;
                     }
-                    Err(e) => tracing::error!(
+                    Err(e) => error!(
                         "Generating rendernodes from images failed while translating stroke images , Err: {e:?}"
                     ),
                 }
@@ -384,7 +386,7 @@ impl StrokeStore {
                     Ok(rendernodes) => {
                         render_comp.rendernodes = rendernodes;
                     }
-                    Err(e) => tracing::error!(
+                    Err(e) => error!(
                         "Generating rendernodes from images failed while rotating stroke images , Err: {e:?}"
                     ),
                 }
@@ -427,7 +429,7 @@ impl StrokeStore {
                     Ok(rendernodes) => {
                         render_comp.rendernodes = rendernodes;
                     }
-                    Err(e) => tracing::error!(
+                    Err(e) => error!(
                         "Generating rendernodes from images failed while scaling stroke images, Err: {e:?}"
                     ),
                 }
@@ -640,6 +642,38 @@ impl StrokeStore {
             .collect::<Vec<StrokeKey>>()
     }
 
+    pub(crate) fn keys_between(
+        &self,
+        y_start: f64,
+        y_end: f64,
+        x_lims: (f64, f64),
+        limit_movement_vertical_border: bool,
+        limit_movement_horizontal_border: bool,
+    ) -> Vec<StrokeKey> {
+        self.key_tree.keys_intersecting_bounds(Aabb::new(
+            na::point![
+                if limit_movement_vertical_border {
+                    x_lims.0
+                } else {
+                    f64::NEG_INFINITY
+                },
+                y_start
+            ],
+            na::point![
+                if limit_movement_vertical_border {
+                    x_lims.1
+                } else {
+                    f64::INFINITY
+                },
+                if limit_movement_horizontal_border {
+                    y_end
+                } else {
+                    f64::INFINITY
+                }
+            ],
+        ))
+    }
+
     pub(crate) fn filter_keys_intersecting_bounds<'a, I: IntoIterator<Item = &'a StrokeKey>>(
         &'a self,
         keys: I,
@@ -684,6 +718,7 @@ impl StrokeStore {
     pub(crate) fn insert_stroke_content(
         &mut self,
         clipboard_content: StrokeContent,
+        ratio: f64,
         pos: na::Vector2<f64>,
     ) -> Vec<StrokeKey> {
         if clipboard_content.strokes.is_empty() {
@@ -700,8 +735,15 @@ impl StrokeStore {
             .map(|s| {
                 let offset = s.bounds().mins.coords - clipboard_bounds.mins.coords;
                 let key = self.insert_stroke((*s).clone(), None);
+                // position strokes without resizing
                 self.set_stroke_pos(key, pos);
                 self.translate_strokes(&[key], offset);
+
+                // apply a rescale around a pivot
+                self.scale_strokes_with_pivot(&[key], na::Vector2::new(ratio, ratio), pos);
+                self.scale_strokes_images_with_pivot(&[key], na::Vector2::new(ratio, ratio), pos);
+
+                // select keys
                 self.set_selected(key, true);
                 key
             })
