@@ -8,9 +8,10 @@ use piet::{RenderContext, TextLayout, TextLayoutBuilder};
 use rnote_compose::ext::{AabbExt, Affine2Ext, Vector2Ext};
 use rnote_compose::shapes::Shapeable;
 use rnote_compose::transform::Transformable;
-use rnote_compose::{color, Color, Transform};
+use rnote_compose::{Color, Transform, color};
 use serde::{Deserialize, Serialize};
 use std::ops::Range;
+use tracing::error;
 use unicode_segmentation::{GraphemeCursor, UnicodeSegmentation};
 
 #[derive(Debug, Clone, Copy, Serialize, Deserialize)]
@@ -129,27 +130,40 @@ impl TextAttribute {
         T: piet::Text,
     {
         match self {
-            TextAttribute::FontFamily(font_family) => piet_text.font_family(font_family.as_str()).map(
-                piet::TextAttribute::FontFamily)
-                    .ok_or_else(|| anyhow::anyhow!("query piet font family failed in TextAttribute try_into_piet() with font family name: {}", font_family)),
+            TextAttribute::FontFamily(font_family) => piet_text
+                .font_family(font_family.as_str())
+                .map(piet::TextAttribute::FontFamily)
+                .ok_or_else(|| {
+                    anyhow::anyhow!(
+                        "query piet font family returned None for font family '{font_family}"
+                    )
+                }),
             TextAttribute::FontSize(font_size) => Ok(piet::TextAttribute::FontSize(font_size)),
-            TextAttribute::FontWeight(font_weight) => Ok(piet::TextAttribute::Weight(piet::FontWeight::new(font_weight))),
-            TextAttribute::TextColor(color) => Ok(piet::TextAttribute::TextColor(piet::Color::from(color))),
-            TextAttribute::Style(style) => Ok(piet::TextAttribute::Style(piet::FontStyle::from(style))),
+            TextAttribute::FontWeight(font_weight) => Ok(piet::TextAttribute::Weight(
+                piet::FontWeight::new(font_weight),
+            )),
+            TextAttribute::TextColor(color) => {
+                Ok(piet::TextAttribute::TextColor(piet::Color::from(color)))
+            }
+            TextAttribute::Style(style) => {
+                Ok(piet::TextAttribute::Style(piet::FontStyle::from(style)))
+            }
             TextAttribute::Underline(underline) => Ok(piet::TextAttribute::Underline(underline)),
-            TextAttribute::Strikethrough(strikethrough) => Ok(piet::TextAttribute::Strikethrough(strikethrough)),
+            TextAttribute::Strikethrough(strikethrough) => {
+                Ok(piet::TextAttribute::Strikethrough(strikethrough))
+            }
         }
     }
 
-    fn is_same_attribute(&self, other: &TextAttribute) -> bool {
+    fn same_variant(&self, other: &TextAttribute) -> bool {
         match (self, other) {
-            (TextAttribute::FontFamily(_), TextAttribute::FontFamily(_)) => true,
-            (TextAttribute::FontSize(_), TextAttribute::FontSize(_)) => true,
-            (TextAttribute::FontWeight(_), TextAttribute::FontWeight(_)) => true,
-            (TextAttribute::Strikethrough(_), TextAttribute::Strikethrough(_)) => true,
-            (TextAttribute::Underline(_), TextAttribute::Underline(_)) => true,
-            (TextAttribute::Style(_), TextAttribute::Style(_)) => true,
-            (TextAttribute::TextColor(_), TextAttribute::TextColor(_)) => true,
+            (TextAttribute::FontFamily(_), TextAttribute::FontFamily(_))
+            | (TextAttribute::FontSize(_), TextAttribute::FontSize(_))
+            | (TextAttribute::FontWeight(_), TextAttribute::FontWeight(_))
+            | (TextAttribute::TextColor(_), TextAttribute::TextColor(_))
+            | (TextAttribute::Style(_), TextAttribute::Style(_))
+            | (TextAttribute::Underline(_), TextAttribute::Underline(_))
+            | (TextAttribute::Strikethrough(_), TextAttribute::Strikethrough(_)) => true,
             (_, _) => false,
         }
     }
@@ -472,9 +486,7 @@ impl Shapeable for TextStroke {
         {
             Ok(text_layout) => text_layout,
             Err(e) => {
-                tracing::error!(
-                    "Building text layout failed while calculating the hitboxes, Err: {e:?}"
-                );
+                error!("Building text layout failed while calculating the hitboxes, Err: {e:?}");
                 return vec![self.bounds()];
             }
         };
@@ -728,13 +740,13 @@ impl TextStroke {
     /// Remove all attributes in the given range.
     pub fn remove_attrs_for_range(&mut self, range: Range<usize>) {
         // partition into attrs that intersect the range, and those who don't and will be retained
-        let (intersecting_attrs, mut retained_attrs) = Self::get_intersecting_attrs_for_range(
+        let (intersecting_attrs, mut retained_attrs) = get_intersecting_attrs_for_range(
             &range,
             self.text_style.ranged_text_attributes.clone(),
         );
 
         // Truncate and filter the ranges of intersecting attrs
-        let truncated_attrs = Self::remove_intersecting_attrs_in_range(&range, intersecting_attrs);
+        let truncated_attrs = remove_intersecting_attrs_in_range(&range, intersecting_attrs);
 
         // Set the updated attributes
         self.text_style.ranged_text_attributes = {
@@ -743,18 +755,40 @@ impl TextStroke {
         };
     }
 
+    /// Replace the attribute of the same type in the given range.
+    pub fn replace_attr_for_range(&mut self, range: Range<usize>, text_attribute: TextAttribute) {
+        let (intersecting_attrs, mut retained_attrs) = get_intersecting_attrs_for_range(
+            &range,
+            self.text_style.ranged_text_attributes.clone(),
+        );
+        let truncated_attrs = remove_intersecting_attrs_in_range(
+            &range,
+            intersecting_attrs
+                .into_iter()
+                .filter(|attr| attr.attribute.same_variant(&text_attribute))
+                .collect(),
+        );
+        self.text_style.ranged_text_attributes = {
+            retained_attrs.extend(truncated_attrs);
+            retained_attrs.push(RangedTextAttribute {
+                range,
+                attribute: text_attribute,
+            });
+            retained_attrs
+        };
+    }
+
     pub fn toggle_attrs_for_range(&mut self, range: Range<usize>, text_attribute: TextAttribute) {
         let (matching_attributes, mut non_matching_attrs) = self
             .text_style
             .ranged_text_attributes
             .clone()
             .into_iter()
-            .partition(|attr| attr.attribute.is_same_attribute(&text_attribute));
+            .partition(|attr| attr.attribute.same_variant(&text_attribute));
 
         let (intersecting_attrs, retained_attrs) =
-            Self::get_intersecting_attrs_for_range(&range, matching_attributes);
+            get_intersecting_attrs_for_range(&range, matching_attributes);
 
-        let _bold_weight = piet::FontWeight::BOLD.to_raw();
         let toggled_attribute = intersecting_attrs
             .clone()
             .into_iter()
@@ -776,60 +810,17 @@ impl TextStroke {
             })
             .unwrap_or_else(|| Some(text_attribute.clone()));
 
-        let truncated_attrs = Self::remove_intersecting_attrs_in_range(&range, intersecting_attrs);
+        let truncated_attrs = remove_intersecting_attrs_in_range(&range, intersecting_attrs);
 
         non_matching_attrs.extend(retained_attrs);
         non_matching_attrs.extend(truncated_attrs);
         if let Some(attribute) = toggled_attribute {
-            non_matching_attrs.extend([RangedTextAttribute { attribute, range }]);
+            non_matching_attrs.push(RangedTextAttribute { attribute, range });
         }
 
-        // Set the updated attributes
         self.text_style.ranged_text_attributes = non_matching_attrs;
     }
 
-    fn get_intersecting_attrs_for_range(
-        range: &Range<usize>,
-        ranged_text_attributes: Vec<RangedTextAttribute>,
-    ) -> (Vec<RangedTextAttribute>, Vec<RangedTextAttribute>) {
-        ranged_text_attributes
-            .into_iter()
-            .partition(|attr| attr.range.end > range.start && attr.range.start < range.end)
-    }
-
-    fn remove_intersecting_attrs_in_range(
-        range: &Range<usize>,
-        intersecting_attrs: Vec<RangedTextAttribute>,
-    ) -> Vec<RangedTextAttribute> {
-        intersecting_attrs
-            .into_iter()
-            .flat_map(|mut attr| {
-                if attr.range.start <= range.start && attr.range.end >= range.end {
-                    // if the attribute completely contains the given range, split it
-                    let mut first_split_attr = attr.clone();
-                    first_split_attr.range.end = range.start;
-                    let mut second_split_attr = attr;
-                    second_split_attr.range.start = range.end;
-
-                    vec![first_split_attr, second_split_attr]
-                } else if attr.range.start <= range.start && attr.range.end > range.start {
-                    // overlapping from the left, so truncate the end
-                    attr.range.end = range.start;
-                    vec![attr]
-                } else if attr.range.end >= range.end && attr.range.start < range.end {
-                    // overlapping from the right, so truncate the start
-                    attr.range.start = range.end;
-                    vec![attr]
-                } else {
-                    // Else the attribute is in the range, so we discard it
-                    vec![]
-                }
-            })
-            // Filter out any that became empty or are contained in the given range
-            .filter(|attr| !attr.range.is_empty())
-            .collect::<Vec<RangedTextAttribute>>()
-    }
-
     pub fn update_selection_entire_text(
         &self,
         cursor: &mut GraphemeCursor,
@@ -849,6 +840,22 @@ impl TextStroke {
         current_char_index
     }
 
+    fn get_prev_word_boundary_index(&self, current_char_index: usize) -> usize {
+        for (start_index, word) in self.text.unicode_word_indices().rev() {
+            let end_index = start_index + word.len();
+
+            if end_index < current_char_index {
+                return end_index;
+            }
+
+            if start_index < current_char_index {
+                return start_index;
+            }
+        }
+
+        current_char_index
+    }
+
     fn get_next_word_end_index(&self, current_char_index: usize) -> usize {
         for (start_index, word) in self.text.unicode_word_indices() {
             let end_index = start_index + word.len();
@@ -861,13 +868,29 @@ impl TextStroke {
         current_char_index
     }
 
+    fn get_next_word_boundary_index(&self, current_char_index: usize) -> usize {
+        for (start_index, word) in self.text.unicode_word_indices() {
+            if start_index >= current_char_index {
+                return start_index;
+            }
+
+            let end_index = start_index + word.len();
+
+            if end_index >= current_char_index {
+                return end_index;
+            }
+        }
+
+        current_char_index
+    }
+
     pub fn move_cursor_back(&self, cursor: &mut GraphemeCursor) {
-        // Cant fail, we are providing the entire text
+        // Can't fail, we are providing the entire text
         cursor.prev_boundary(&self.text, 0).unwrap();
     }
 
     pub fn move_cursor_forward(&self, cursor: &mut GraphemeCursor) {
-        // Cant fail, we are providing the entire text
+        // Can't fail, we are providing the entire text
         cursor.next_boundary(&self.text, 0).unwrap();
     }
 
@@ -875,10 +898,18 @@ impl TextStroke {
         cursor.set_cursor(self.get_prev_word_start_index(cursor.cur_cursor()));
     }
 
+    pub fn move_cursor_word_boundary_back(&self, cursor: &mut GraphemeCursor) {
+        cursor.set_cursor(self.get_prev_word_boundary_index(cursor.cur_cursor()));
+    }
+
     pub fn move_cursor_word_forward(&self, cursor: &mut GraphemeCursor) {
         cursor.set_cursor(self.get_next_word_end_index(cursor.cur_cursor()));
     }
 
+    pub fn move_cursor_word_boundary_forward(&self, cursor: &mut GraphemeCursor) {
+        cursor.set_cursor(self.get_next_word_boundary_index(cursor.cur_cursor()));
+    }
+
     pub fn move_cursor_text_start(&self, cursor: &mut GraphemeCursor) {
         cursor.set_cursor(0);
     }
@@ -888,49 +919,41 @@ impl TextStroke {
     }
 
     pub fn move_cursor_line_start(&self, cursor: &mut GraphemeCursor) {
-        if let (Ok(lines), Ok(hittest_position)) = (
-            self.text_style
-                .lines(&mut piet_cairo::CairoText::new(), self.text.clone()),
-            self.text_style.cursor_hittest_position(
+        if let Ok(lines) = self
+            .text_style
+            .lines(&mut piet_cairo::CairoText::new(), self.text.clone())
+            && let Ok(hittest_position) = self.text_style.cursor_hittest_position(
                 &mut piet_cairo::CairoText::new(),
                 self.text.clone(),
                 cursor,
-            ),
-        ) {
+            )
+        {
             cursor.set_cursor(lines[hittest_position.line].start_offset);
         }
     }
 
     pub fn move_cursor_line_end(&self, cursor: &mut GraphemeCursor) {
-        if let (Ok(lines), Ok(hittest_position)) = (
-            self.text_style
-                .lines(&mut piet_cairo::CairoText::new(), self.text.clone()),
-            self.text_style.cursor_hittest_position(
+        if let Ok(lines) = self
+            .text_style
+            .lines(&mut piet_cairo::CairoText::new(), self.text.clone())
+            && let Ok(hittest_position) = self.text_style.cursor_hittest_position(
                 &mut piet_cairo::CairoText::new(),
                 self.text.clone(),
                 cursor,
-            ),
-        ) {
+            )
+        {
             let line_metric = &lines[hittest_position.line];
             let mut offset = line_metric.end_offset;
 
             // Move cursor in front of new line characters if they exist.
             if offset > line_metric.start_offset
-                && self
-                    .text
-                    .chars()
-                    .nth(offset - 1)
-                    .map_or(false, |c| c == '\n')
+                && (self.text.chars().nth(offset - 1) == Some('\n'))
             {
                 offset -= 1;
             }
 
             if offset > line_metric.start_offset
-                && self
-                    .text
-                    .chars()
-                    .nth(offset - 1)
-                    .map_or(false, |c| c == '\r')
+                && (self.text.chars().nth(offset - 1) == Some('\r'))
             {
                 offset -= 1;
             }
@@ -940,17 +963,18 @@ impl TextStroke {
     }
 
     pub fn move_cursor_line_down(&self, cursor: &mut GraphemeCursor) {
-        if let (Ok(text_layout), Ok(lines), Ok(hittest_position)) = (
-            self.text_style
-                .build_text_layout(&mut piet_cairo::CairoText::new(), self.text.clone()),
-            self.text_style
-                .lines(&mut piet_cairo::CairoText::new(), self.text.clone()),
-            self.text_style.cursor_hittest_position(
+        if let Ok(text_layout) = self
+            .text_style
+            .build_text_layout(&mut piet_cairo::CairoText::new(), self.text.clone())
+            && let Ok(lines) = self
+                .text_style
+                .lines(&mut piet_cairo::CairoText::new(), self.text.clone())
+            && let Ok(hittest_position) = self.text_style.cursor_hittest_position(
                 &mut piet_cairo::CairoText::new(),
                 self.text.clone(),
                 cursor,
-            ),
-        ) {
+            )
+        {
             let next_line = (hittest_position.line + 1).min(lines.len().saturating_sub(1));
 
             if next_line != hittest_position.line {
@@ -967,17 +991,18 @@ impl TextStroke {
     }
 
     pub fn move_cursor_line_up(&self, cursor: &mut GraphemeCursor) {
-        if let (Ok(text_layout), Ok(lines), Ok(hittest_position)) = (
-            self.text_style
-                .build_text_layout(&mut piet_cairo::CairoText::new(), self.text.clone()),
-            self.text_style
-                .lines(&mut piet_cairo::CairoText::new(), self.text.clone()),
-            self.text_style.cursor_hittest_position(
+        if let Ok(text_layout) = self
+            .text_style
+            .build_text_layout(&mut piet_cairo::CairoText::new(), self.text.clone())
+            && let Ok(lines) = self
+                .text_style
+                .lines(&mut piet_cairo::CairoText::new(), self.text.clone())
+            && let Ok(hittest_position) = self.text_style.cursor_hittest_position(
                 &mut piet_cairo::CairoText::new(),
                 self.text.clone(),
                 cursor,
-            ),
-        ) {
+            )
+        {
             let prev_line = hittest_position.line.saturating_sub(1);
 
             if prev_line != hittest_position.line {
@@ -991,3 +1016,45 @@ impl TextStroke {
         }
     }
 }
+
+fn get_intersecting_attrs_for_range(
+    range: &Range<usize>,
+    ranged_text_attributes: Vec<RangedTextAttribute>,
+) -> (Vec<RangedTextAttribute>, Vec<RangedTextAttribute>) {
+    ranged_text_attributes
+        .into_iter()
+        .partition(|attr| attr.range.end > range.start && attr.range.start < range.end)
+}
+
+fn remove_intersecting_attrs_in_range(
+    range: &Range<usize>,
+    intersecting_attrs: Vec<RangedTextAttribute>,
+) -> Vec<RangedTextAttribute> {
+    intersecting_attrs
+        .into_iter()
+        .flat_map(|mut attr| {
+            if attr.range.start <= range.start && attr.range.end >= range.end {
+                // if the attribute completely contains the given range, split it
+                let mut first_split_attr = attr.clone();
+                first_split_attr.range.end = range.start;
+                let mut second_split_attr = attr;
+                second_split_attr.range.start = range.end;
+
+                vec![first_split_attr, second_split_attr]
+            } else if attr.range.start <= range.start && attr.range.end > range.start {
+                // overlapping from the left, so truncate the end
+                attr.range.end = range.start;
+                vec![attr]
+            } else if attr.range.end >= range.end && attr.range.start < range.end {
+                // overlapping from the right, so truncate the start
+                attr.range.start = range.end;
+                vec![attr]
+            } else {
+                // Else the attribute is in the range, so we discard it
+                vec![]
+            }
+        })
+        // Filter out any that became empty or are contained in the given range
+        .filter(|attr| !attr.range.is_empty())
+        .collect::<Vec<RangedTextAttribute>>()
+}
