@@ -2,33 +2,36 @@
 mod canvaslayout;
 pub(crate) mod imexport;
 mod input;
-mod widgetflagsboxed;
 
 // Re-exports
 pub(crate) use canvaslayout::RnCanvasLayout;
-pub(crate) use widgetflagsboxed::WidgetFlagsBoxed;
+pub(crate) use input::reject_pointer_input;
 
 // Imports
-use crate::{config, RnAppWindow};
+use crate::boxed::WidgetFlagsBoxed;
+use crate::{RnAppWindow, config, env};
 use futures::StreamExt;
 use gettextrs::gettext;
 use gtk4::{
-    gdk, gio, glib, glib::clone, graphene, prelude::*, subclass::prelude::*, Adjustment,
-    DropTarget, EventControllerKey, EventControllerLegacy, IMMulticontext, PropagationPhase,
-    Scrollable, ScrollablePolicy, Widget,
+    Adjustment, DropTarget, EventControllerKey, EventControllerLegacy, IMMulticontext,
+    PropagationPhase, Scrollable, ScrollablePolicy, Widget, gdk, gio, glib, glib::clone, graphene,
+    prelude::*, subclass::prelude::*,
 };
-use notify_debouncer_full::notify::{self, Watcher};
+use notify::EventKind;
+use notify::event::{AccessKind, AccessMode, ModifyKind, RenameMode};
+use notify_debouncer_full::notify;
 use once_cell::sync::Lazy;
 use p2d::bounding_volume::Aabb;
 use rnote_compose::ext::AabbExt;
 use rnote_compose::penevent::PenState;
+use rnote_engine::Camera;
 use rnote_engine::ext::GraphenePointExt;
 use rnote_engine::ext::GrapheneRectExt;
-use rnote_engine::Camera;
 use rnote_engine::{Engine, WidgetFlags};
 use std::cell::{Cell, Ref, RefCell, RefMut};
 use std::path::Path;
 use std::time::Duration;
+use tracing::{debug, error, warn};
 
 #[derive(Debug, Default)]
 struct Connections {
@@ -36,8 +39,10 @@ struct Connections {
     vadjustment: Option<glib::SignalHandlerId>,
     tab_page_output_file: Option<glib::Binding>,
     tab_page_unsaved_changes: Option<glib::Binding>,
+    tab_page_invalidate_thumbnail: Option<glib::SignalHandlerId>,
     appwindow_output_file: Option<glib::SignalHandlerId>,
     appwindow_scalefactor: Option<glib::SignalHandlerId>,
+    appwindow_save_in_progress: Option<glib::SignalHandlerId>,
     appwindow_unsaved_changes: Option<glib::SignalHandlerId>,
     appwindow_touch_drawing: Option<glib::Binding>,
     appwindow_show_drawing_cursor: Option<glib::Binding>,
@@ -70,10 +75,11 @@ mod imp {
 
         pub(crate) engine: RefCell<Engine>,
         pub(crate) engine_task_handler_handle: RefCell<Option<glib::JoinHandle<()>>>,
+        pub(crate) animation_callback_id: RefCell<Option<gtk4::TickCallbackId>>,
 
         pub(crate) output_file: RefCell<Option<gio::File>>,
         pub(crate) output_file_watcher_task: RefCell<Option<glib::JoinHandle<()>>>,
-        pub(crate) output_file_modified_toast_singleton: RefCell<Option<adw::Toast>>,
+        pub(crate) output_file_modified_toast_singleton: glib::WeakRef<adw::Toast>,
         pub(crate) output_file_expect_write: Cell<bool>,
         pub(crate) save_in_progress: Cell<bool>,
         pub(crate) unsaved_changes: Cell<bool>,
@@ -163,11 +169,12 @@ mod imp {
 
                 engine: RefCell::new(engine),
                 engine_task_handler_handle: RefCell::new(None),
+                animation_callback_id: RefCell::new(None),
 
                 output_file: RefCell::new(None),
                 output_file_watcher_task: RefCell::new(None),
                 // is automatically updated whenever the output file changes.
-                output_file_modified_toast_singleton: RefCell::new(None),
+                output_file_modified_toast_singleton: glib::WeakRef::new(),
                 output_file_expect_write: Cell::new(false),
                 save_in_progress: Cell::new(false),
                 unsaved_changes: Cell::new(false),
@@ -214,10 +221,14 @@ mod imp {
             obj.add_controller(self.drop_target.clone());
 
             // receive and handle engine tasks
-            let engine_task_handler_handle = glib::spawn_future_local(
-                clone!(@weak obj as canvas => async move {
+            let engine_task_handler_handle = glib::spawn_future_local(clone!(
+                #[weak(rename_to=canvas)]
+                obj,
+                async move {
                     let Some(mut task_rx) = canvas.engine_mut().take_engine_tasks_rx() else {
-                        tracing::error!("Installing the engine task handler failed, taken tasks_rx is None.");
+                        error!(
+                            "Installing the engine task handler failed, taken tasks_rx is None."
+                        );
                         return;
                     };
 
@@ -231,11 +242,34 @@ mod imp {
                             }
                         }
                     }
-                }),
-            );
+                }
+            ));
 
             *self.engine_task_handler_handle.borrow_mut() = Some(engine_task_handler_handle);
 
+            let animation_callback_id = obj.add_tick_callback(clone!(
+                #[weak(rename_to=canvas)]
+                obj,
+                #[upgrade_or]
+                glib::ControlFlow::Break,
+                move |_widget, _frame_clock| {
+                    if canvas.engine_mut().animation.process_frame() {
+                        let optimize_epd = canvas.engine_ref().optimize_epd();
+                        canvas.engine_mut().handle_animation_frame();
+
+                        // if optimize_epd is enabled, we only redraw the canvas
+                        // when no follow-up frame has been requested (i.e. the animation is done)
+                        if !optimize_epd || !canvas.engine_ref().animation.frame_in_flight() {
+                            canvas.queue_draw();
+                        }
+                    }
+
+                    glib::ControlFlow::Continue
+                }
+            ));
+
+            *self.animation_callback_id.borrow_mut() = Some(animation_callback_id);
+
             self.setup_input();
         }
 
@@ -253,6 +287,9 @@ mod imp {
                 vec![
                     // this is nullable, so it can be used to represent Option<gio::File>
                     glib::ParamSpecObject::builder::<gio::File>("output-file").build(),
+                    glib::ParamSpecBoolean::builder("save-in-progress")
+                        .default_value(false)
+                        .build(),
                     glib::ParamSpecBoolean::builder("unsaved-changes")
                         .default_value(false)
                         .build(),
@@ -284,6 +321,7 @@ mod imp {
         fn property(&self, _id: usize, pspec: &glib::ParamSpec) -> glib::Value {
             match pspec.name() {
                 "output-file" => self.output_file.borrow().to_value(),
+                "save-in-progress" => self.save_in_progress.get().to_value(),
                 "unsaved-changes" => self.unsaved_changes.get().to_value(),
                 "empty" => self.empty.get().to_value(),
                 "hadjustment" => self.hadjustment.borrow().to_value(),
@@ -308,6 +346,11 @@ mod imp {
                         .expect("The value needs to be of type `Option<gio::File>`");
                     self.output_file.replace(output_file);
                 }
+                "save-in-progress" => {
+                    let save_in_progress: bool =
+                        value.get().expect("The value needs to be of type `bool`");
+                    self.save_in_progress.replace(save_in_progress);
+                }
                 "unsaved-changes" => {
                     let unsaved_changes: bool =
                         value.get().expect("The value needs to be of type `bool`");
@@ -404,9 +447,12 @@ mod imp {
 
         fn signals() -> &'static [glib::subclass::Signal] {
             static SIGNALS: Lazy<Vec<glib::subclass::Signal>> = Lazy::new(|| {
-                vec![glib::subclass::Signal::builder("handle-widget-flags")
-                    .param_types([WidgetFlagsBoxed::static_type()])
-                    .build()]
+                vec![
+                    glib::subclass::Signal::builder("handle-widget-flags")
+                        .param_types([WidgetFlagsBoxed::static_type()])
+                        .build(),
+                    glib::subclass::Signal::builder("invalidate-thumbnail").build(),
+                ]
             });
             SIGNALS.as_ref()
         }
@@ -442,7 +488,7 @@ mod imp {
                 snapshot.pop();
                 Ok(())
             }() {
-                tracing::error!("Snapshot canvas failed , Err: {e:?}");
+                error!("Snapshot canvas failed , Err: {e:?}");
             }
         }
     }
@@ -455,29 +501,51 @@ mod imp {
 
             // Pointer controller
             let pen_state = Cell::new(PenState::Up);
-            self.pointer_controller.connect_event(clone!(@strong pen_state, @weak obj as canvas => @default-return glib::Propagation::Proceed, move |_, event| {
-                let (propagation, new_state) = super::input::handle_pointer_controller_event(&canvas, event, pen_state.get());
-                pen_state.set(new_state);
-                propagation
-            }));
+            self.pointer_controller.connect_event(clone!(
+                #[strong]
+                pen_state,
+                #[weak(rename_to=canvas)]
+                obj,
+                #[upgrade_or]
+                glib::Propagation::Proceed,
+                move |_, event| {
+                    let (propagation, new_state) = super::input::handle_pointer_controller_event(
+                        &canvas,
+                        event,
+                        pen_state.get(),
+                    );
+                    pen_state.set(new_state);
+                    propagation
+                }
+            ));
 
             // For unicode text the input is committed from the IM context, and won't trigger the key_pressed signal
-            self.key_controller_im_context.connect_commit(
-                clone!(@weak obj as canvas => move |_cx, text| {
+            self.key_controller_im_context.connect_commit(clone!(
+                #[weak(rename_to=canvas)]
+                obj,
+                move |_cx, text| {
                     super::input::handle_imcontext_text_commit(&canvas, text);
-                }),
-            );
+                }
+            ));
 
             // Key controller
-            self.key_controller.connect_key_pressed(clone!(@weak obj as canvas => @default-return glib::Propagation::Proceed, move |_, key, _raw, modifier| {
-                super::input::handle_key_controller_key_pressed(&canvas, key, modifier)
-            }));
+            self.key_controller.connect_key_pressed(clone!(
+                #[weak(rename_to=canvas)]
+                obj,
+                #[upgrade_or]
+                glib::Propagation::Proceed,
+                move |_, key, _raw, modifier| {
+                    super::input::handle_key_controller_key_pressed(&canvas, key, modifier)
+                }
+            ));
 
-            self.key_controller.connect_key_released(
-                clone!(@weak obj as canvas => move |_key_controller, key, _raw, modifier| {
+            self.key_controller.connect_key_released(clone!(
+                #[weak(rename_to=canvas)]
+                obj,
+                move |_key_controller, key, _raw, modifier| {
                     super::input::handle_key_controller_key_released(&canvas, key, modifier)
-                }),
-            );
+                }
+            ));
         }
 
         fn set_hadjustment_prop(&self, hadj: Option<Adjustment>) {
@@ -504,12 +572,15 @@ mod imp {
             }
 
             if let Some(ref hadj) = hadj {
-                let signal_id =
-                    hadj.connect_value_changed(clone!(@weak obj as canvas => move |_| {
+                let signal_id = hadj.connect_value_changed(clone!(
+                    #[weak(rename_to=canvas)]
+                    obj,
+                    move |_| {
                         // this triggers a canvaslayout allocate() call,
                         // where the camera and content rendering is updated based on some conditions
                         canvas.queue_resize();
-                    }));
+                    }
+                ));
 
                 self.connections.borrow_mut().hadjustment.replace(signal_id);
             }
@@ -546,12 +617,15 @@ mod imp {
             }
 
             if let Some(ref vadj) = vadj {
-                let signal_id =
-                    vadj.connect_value_changed(clone!(@weak obj as canvas => move |_| {
+                let signal_id = vadj.connect_value_changed(clone!(
+                    #[weak(rename_to=canvas)]
+                    obj,
+                    move |_| {
                         // this triggers a canvaslayout allocate() call,
                         // where the camera and content rendering is updated based on some conditions
                         canvas.queue_resize();
-                    }));
+                    }
+                ));
 
                 self.connections.borrow_mut().vadjustment.replace(signal_id);
             }
@@ -568,7 +642,7 @@ mod imp {
 
 glib::wrapper! {
     pub(crate) struct RnCanvas(ObjectSubclass<imp::RnCanvas>)
-        @extends gtk4::Widget,
+        @extends Widget,
         @implements gtk4::Accessible, gtk4::Buildable, gtk4::ConstraintTarget, gtk4::Scrollable;
 }
 
@@ -633,12 +707,14 @@ impl RnCanvas {
 
     #[allow(unused)]
     pub(crate) fn save_in_progress(&self) -> bool {
-        self.imp().save_in_progress.get()
+        self.property::<bool>("save-in-progress")
     }
 
     #[allow(unused)]
     pub(crate) fn set_save_in_progress(&self, save_in_progress: bool) {
-        self.imp().save_in_progress.set(save_in_progress);
+        if self.imp().save_in_progress.get() != save_in_progress {
+            self.set_property("save-in-progress", save_in_progress.to_value());
+        }
     }
 
     #[allow(unused)]
@@ -697,6 +773,10 @@ impl RnCanvas {
         );
     }
 
+    pub(super) fn emit_invalidate_thumbnail(&self) {
+        self.emit_by_name::<()>("invalidate-thumbnail", &[]);
+    }
+
     pub(crate) fn last_export_dir(&self) -> Option<gio::File> {
         self.imp().last_export_dir.borrow().clone()
     }
@@ -751,12 +831,12 @@ impl RnCanvas {
     }
 
     /// Immutable borrow of the engine.
-    pub(crate) fn engine_ref(&self) -> Ref<Engine> {
+    pub(crate) fn engine_ref(&self) -> Ref<'_, Engine> {
         self.imp().engine.borrow()
     }
 
     /// Mutable borrow of the engine.
-    pub(crate) fn engine_mut(&self) -> RefMut<Engine> {
+    pub(crate) fn engine_mut(&self) -> RefMut<'_, Engine> {
         self.imp().engine.borrow_mut()
     }
 
@@ -783,40 +863,6 @@ impl RnCanvas {
         }
     }
 
-    pub(crate) fn save_engine_config(&self, settings: &gio::Settings) -> anyhow::Result<()> {
-        let engine_config = self.engine_ref().export_engine_config_as_json()?;
-        Ok(settings.set_string("engine-config", engine_config.as_str())?)
-    }
-
-    pub(crate) fn load_engine_config_from_settings(
-        &self,
-        settings: &gio::Settings,
-    ) -> anyhow::Result<()> {
-        // load engine config
-        let engine_config = settings.string("engine-config");
-        let widget_flags = match self
-            .engine_mut()
-            .import_engine_config_from_json(&engine_config, crate::env::pkg_data_dir().ok())
-        {
-            Err(e) => {
-                if engine_config.is_empty() {
-                    // On first app startup the engine config is empty, so we don't log an error
-                    tracing::debug!("Did not load `engine-config` from settings, was empty");
-                } else {
-                    return Err(e);
-                }
-                None
-            }
-            Ok(widget_flags) => Some(widget_flags),
-        };
-
-        // Avoiding already borrowed
-        if let Some(widget_flags) = widget_flags {
-            self.emit_handle_widget_flags(widget_flags);
-        }
-        Ok(())
-    }
-
     /// Switches between the regular and the drawing cursor
     pub(crate) fn enable_drawing_cursor(&self, drawing_cursor: bool) {
         if drawing_cursor == self.imp().drawing_cursor_enabled.get() {
@@ -869,7 +915,7 @@ impl RnCanvas {
 
     pub(crate) fn dismiss_output_file_modified_toast(&self) {
         if let Some(output_file_modified_toast) =
-            self.imp().output_file_modified_toast_singleton.take()
+            self.imp().output_file_modified_toast_singleton.upgrade()
         {
             output_file_modified_toast.dismiss();
         }
@@ -880,33 +926,44 @@ impl RnCanvas {
             canvas.set_unsaved_changes(true);
 
             appwindow.overlays().dispatch_toast_w_button_singleton(
-                        &gettext("Opened file was modified on disk"),
-                        &gettext("Reload"),
-                        clone!(@weak canvas, @weak appwindow => move |_reload_toast| {
-                            glib::spawn_future_local(clone!(@weak appwindow => async move {
+                &gettext("Opened file was modified on disk"),
+                &gettext("Reload"),
+                clone!(
+                    #[weak]
+                    canvas,
+                    #[weak]
+                    appwindow,
+                    move |_reload_toast| {
+                        glib::spawn_future_local(clone!(
+                            #[weak]
+                            appwindow,
+                            async move {
                                 appwindow.overlays().progressbar_start_pulsing();
 
                                 if let Err(e) = canvas.reload_from_disk().await {
-                                    tracing::error!("Failed to reload current output file, Err: {e:?}");
-                                    appwindow.overlays().dispatch_toast_error(&gettext("Reloading .rnote file from disk failed"));
+                                    error!("Failed to reload current output file, Err: {e:?}");
+                                    appwindow.overlays().dispatch_toast_error(&gettext(
+                                        "Reloading .rnote file from disk failed",
+                                    ));
                                     appwindow.overlays().progressbar_abort();
                                 } else {
                                     appwindow.overlays().progressbar_finish();
+                                    canvas.emit_invalidate_thumbnail();
                                 }
-                            }));
-                        }),
-                        0,
-                    &mut canvas.imp().output_file_modified_toast_singleton.borrow_mut());
+                            }
+                        ));
+                    }
+                ),
+                None,
+                &canvas.imp().output_file_modified_toast_singleton,
+            );
         };
 
         let event_handler = move |appwindow: &RnAppWindow,
                                   canvas: &RnCanvas,
                                   event: notify_debouncer_full::DebouncedEvent,
                                   file_path: &Path| {
-            use notify::event::{AccessKind, AccessMode, ModifyKind, RenameMode};
-            use notify::EventKind;
-
-            tracing::trace!("file parent directory watcher - received event: {event:?}");
+            debug!(?event, expect_write=?canvas.output_file_expect_write(), msg="output file parent directory watcher event received");
 
             match event.kind {
                 EventKind::Create(_create_kind) => {}
@@ -988,35 +1045,47 @@ impl RnCanvas {
             }
         };
 
-        let new_watcher_task = glib::spawn_future_local(
-            glib::clone!(@strong file, @weak self as canvas, @weak appwindow => async move {
+        let new_watcher_task = glib::spawn_future_local(glib::clone!(
+            #[strong]
+            file,
+            #[weak(rename_to=canvas)]
+            self,
+            #[weak]
+            appwindow,
+            async move {
                 let (tx, mut rx) = futures::channel::mpsc::unbounded();
                 let Some(file_path) = file.path() else {
-                    tracing::warn!("Can't create watcher for file that has no path");
+                    warn!("Can't create watcher for file that has no path");
                     return;
                 };
                 let Some(parent_path) = file_path.parent() else {
-                    tracing::warn!("Can't create watcher for file that has no parent directory");
+                    warn!("Can't create watcher for file that has no parent directory");
                     return;
                 };
 
-                let mut debouncer = match notify_debouncer_full::new_debouncer(Duration::from_millis(1000), None, move |res| {
-                    if let Err(e) = tx.unbounded_send(res) {
-                        tracing::error!("File watcher reported change, but failed to send it through channel. Err: {e:?}");
-                    }
-                }) {
-                    Ok(w) => {
-                        w
+                let mut debouncer = match notify_debouncer_full::new_debouncer(
+                    Duration::from_millis(1000),
+                    None,
+                    move |res| {
+                        if let Err(e) = tx.unbounded_send(res) {
+                            error!(
+                                "File watcher reported change, but failed to send it through channel. Err: {e:?}"
+                            );
+                        }
                     },
+                ) {
+                    Ok(w) => w,
                     Err(e) => {
-                        tracing::error!("Failed to create file watcher, Err: {e:?}");
+                        error!("Failed to create file watcher, Err: {e:?}");
                         return;
                     }
                 };
-                if let Err(e) = debouncer.watcher().watch(parent_path, notify::RecursiveMode::NonRecursive) {
-                    tracing::error!("Failed to start watching directory '{}', Err: {e:?}", parent_path.display());
+                if let Err(e) = debouncer.watch(parent_path, notify::RecursiveMode::NonRecursive) {
+                    error!(
+                        "Failed to start watching directory '{}', Err: {e:?}",
+                        parent_path.display()
+                    );
                 }
-                debouncer.cache().add_root(parent_path, notify::RecursiveMode::NonRecursive);
                 while let Some(res) = rx.next().await {
                     match res {
                         Ok(events) => {
@@ -1024,11 +1093,11 @@ impl RnCanvas {
                                 event_handler(&appwindow, &canvas, event, &file_path);
                             }
                         }
-                        Err(e) => tracing::error!("File watcher sent error message, Err: {e:?}"),
+                        Err(e) => error!("File watcher sent error message, Err: {e:?}"),
                     }
                 }
-            }),
-        );
+            }
+        ));
 
         if let Some(old_watcher_task) = self
             .imp()
@@ -1057,16 +1126,19 @@ impl RnCanvas {
 
         let appwindow_output_file = self.connect_notify_local(
             Some("output-file"),
-            clone!(@weak appwindow => move |canvas, _pspec| {
-                if let Some(output_file) = canvas.output_file(){
-                    canvas.create_output_file_watcher(&output_file, &appwindow);
-                } else {
-                    canvas.clear_output_file_watcher();
-                    canvas.dismiss_output_file_modified_toast();
+            clone!(
+                #[weak]
+                appwindow,
+                move |canvas, _pspec| {
+                    if let Some(output_file) = canvas.output_file() {
+                        canvas.create_output_file_watcher(&output_file, &appwindow);
+                    } else {
+                        canvas.clear_output_file_watcher();
+                        canvas.dismiss_output_file_modified_toast();
+                    }
+                    appwindow.refresh_titles(canvas);
                 }
-
-                appwindow.refresh_titles(&appwindow.active_tab_wrapper());
-            }),
+            ),
         );
 
         // set scale factor initially
@@ -1082,12 +1154,32 @@ impl RnCanvas {
                 canvas.emit_handle_widget_flags(widget_flags);
             });
 
+        // Reset
+        let appwindow_save_in_progress = self.connect_notify_local(
+            Some("save-in-progress"),
+            clone!(
+                #[weak]
+                appwindow,
+                move |canvas, _| {
+                    if canvas.save_in_progress() {
+                        appwindow.set_save_in_progress(true);
+                    } else if !appwindow.tabs_any_saves_in_progress() {
+                        appwindow.set_save_in_progress(false);
+                    }
+                }
+            ),
+        );
+
         // Update titles when there are changes
         let appwindow_unsaved_changes = self.connect_notify_local(
             Some("unsaved-changes"),
-            clone!(@weak appwindow => move |_canvas, _pspec| {
-                appwindow.refresh_titles(&appwindow.active_tab_wrapper());
-            }),
+            clone!(
+                #[weak]
+                appwindow,
+                move |canvas, _| {
+                    appwindow.refresh_titles(canvas);
+                }
+            ),
         );
 
         // one per-appwindow property for touch-drawing
@@ -1125,53 +1217,75 @@ impl RnCanvas {
             .build();
 
         // Drop Target
-        let appwindow_drop_target = self.imp().drop_target.connect_drop(
-            clone!(@weak self as canvas, @weak appwindow => @default-return false, move |_, value, x, y| {
-                let pos = (canvas.engine_ref().camera.transform().inverse() *
-                    na::point![x,y]).coords;
+        let appwindow_drop_target = self.imp().drop_target.connect_drop(clone!(
+            #[weak(rename_to=canvas)]
+            self,
+            #[weak]
+            appwindow,
+            #[upgrade_or]
+            false,
+            move |_, value, x, y| {
+                let pos =
+                    (canvas.engine_ref().camera.transform().inverse() * na::point![x, y]).coords;
                 let mut accept_drop = false;
 
                 if value.is::<gio::File>() {
                     // In some scenarios, get() can fail with `UnexpectedNone` even though is() returned true, e.g. when dealing with trashed files.
                     match value.get::<gio::File>() {
                         Ok(file) => {
-                            glib::spawn_future_local(clone!(@weak appwindow => async move {
-                                appwindow.open_file_w_dialogs(file, Some(pos), true).await;
-                            }));
+                            glib::spawn_future_local(clone!(
+                                #[weak]
+                                appwindow,
+                                async move {
+                                    appwindow.open_file_w_dialogs(file, Some(pos), true).await;
+                                }
+                            ));
                             accept_drop = true;
-                        },
+                        }
                         Err(e) => {
-                            tracing::error!("Failed to get dropped in file, Err: {e:?}");
-                            appwindow.overlays().dispatch_toast_error(&gettext("Inserting file failed"));
-                        },
+                            error!("Failed to get dropped in file, Err: {e:?}");
+                            appwindow
+                                .overlays()
+                                .dispatch_toast_error(&gettext("Inserting file failed"));
+                        }
                     };
                 } else if value.is::<String>() {
                     match canvas.load_in_text(value.get::<String>().unwrap(), Some(pos)) {
                         Ok(_) => {
                             accept_drop = true;
-                        },
+                        }
                         Err(e) => {
-                            tracing::error!("Failed to insert dropped in text, Err: {e:?}");
-                            appwindow.overlays().dispatch_toast_error(&gettext("Inserting text failed"));
+                            error!("Failed to insert dropped in text, Err: {e:?}");
+                            appwindow
+                                .overlays()
+                                .dispatch_toast_error(&gettext("Inserting text failed"));
                         }
                     };
                 }
 
                 accept_drop
-            }),
-        );
+            }
+        ));
 
         // handle widget flags
         let appwindow_handle_widget_flags = self.connect_local(
             "handle-widget-flags",
             false,
-            clone!(@weak self as canvas, @weak appwindow => @default-return None, move |args| {
-                // first argument is the widget, second is widget flags
-                let widget_flags = args[1].get::<WidgetFlagsBoxed>().unwrap().inner();
+            clone!(
+                #[weak(rename_to=canvas)]
+                self,
+                #[weak]
+                appwindow,
+                #[upgrade_or]
+                None,
+                move |args| {
+                    // first argument is the widget, second is widget flags
+                    let widget_flags = args[1].get::<WidgetFlagsBoxed>().unwrap().inner();
 
-                appwindow.handle_widget_flags(widget_flags, &canvas);
-                None
-            }),
+                    appwindow.handle_widget_flags(widget_flags, &canvas);
+                    None
+                }
+            ),
         );
 
         // Replace connections
@@ -1188,6 +1302,12 @@ impl RnCanvas {
         {
             self.disconnect(old);
         }
+        if let Some(old) = connections
+            .appwindow_save_in_progress
+            .replace(appwindow_save_in_progress)
+        {
+            self.disconnect(old);
+        }
         if let Some(old) = connections
             .appwindow_unsaved_changes
             .replace(appwindow_unsaved_changes)
@@ -1230,6 +1350,11 @@ impl RnCanvas {
         {
             self.disconnect(old);
         }
+
+        let widget_flags = self
+            .engine_mut()
+            .install_config(appwindow.engine_config(), env::pkg_data_dir().ok());
+        appwindow.handle_widget_flags(widget_flags, self);
     }
 
     /// Disconnect all connections with references to external objects
@@ -1245,6 +1370,9 @@ impl RnCanvas {
         if let Some(old) = connections.appwindow_scalefactor.take() {
             self.disconnect(old);
         }
+        if let Some(old) = connections.appwindow_save_in_progress.take() {
+            self.disconnect(old);
+        }
         if let Some(old) = connections.appwindow_unsaved_changes.take() {
             self.disconnect(old);
         }
@@ -1274,6 +1402,9 @@ impl RnCanvas {
         if let Some(old) = connections.tab_page_unsaved_changes.take() {
             old.unbind();
         }
+        if let Some(old) = connections.tab_page_invalidate_thumbnail.take() {
+            self.disconnect(old);
+        }
     }
 
     /// When the widget is the child of a tab page, we want to connect their titles, icons, ..
@@ -1303,6 +1434,22 @@ impl RnCanvas {
             .sync_create()
             .build();
 
+        // handle invalidating cached thumbnail in the tabs overview panel
+        let tab_page_invalidate_thumbnail = self.connect_local(
+            "invalidate-thumbnail",
+            false,
+            clone!(
+                #[weak]
+                page,
+                #[upgrade_or]
+                None,
+                move |_| {
+                    page.invalidate_thumbnail();
+                    None
+                }
+            ),
+        );
+
         let mut connections = self.imp().connections.borrow_mut();
         if let Some(old) = connections
             .tab_page_output_file
@@ -1316,6 +1463,12 @@ impl RnCanvas {
         {
             old.unbind();
         }
+        if let Some(old) = connections
+            .tab_page_invalidate_thumbnail
+            .replace(tab_page_invalidate_thumbnail)
+        {
+            self.disconnect(old);
+        }
     }
 
     pub(crate) fn bounds(&self) -> Aabb {
